      REAL FUNCTION alog1p(x)
C     (alog1p)
C     Return alog(1 + xs), taking care to avoid subtraction loss.
C
C     This version does internal computations in double precision.
C     (17-Jun-2002)
      REAL x
      DOUBLE PRECISION dlog1p
      alog1p = sngl(dlog1p(dble(x)))
      END

      DOUBLE PRECISION FUNCTION dlog1p(x)
C     (dlog1p)
C     Return dlog(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
      DOUBLE PRECISION sum, term, xn, xtons
C
C     CUTLO = -1/beta, for arbitrary base beta
C
      DATA CUTLO / -0.5d+00 /
C
C     CUTHI = 1/beta, for arbitrary base beta
C
      DATA CUTHI / +0.5d+00 /
C
      DATA ONE   / 1.0d+00 /
      DATA ZERO  / 0.0d+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  log(1+x)
C     x in [CUTLO, CUTHI]:      Taylor series
C     x in (CUTHI, Infinity]:   log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of dlog1p(NaN) and dlog1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          dlog1p = x
      ELSE IF (x .eq. ZERO) THEN
          dlog1p = x
      ELSE IF (x .lt. CUTLO) THEN
          dlog1p = dlog(ONE + x)
      ELSE IF (x .le. CUTHI) THEN
          term = x
          sum = ZERO
          xn = ONE
          xtons = x
   10     IF ((sum + term) .ne. sum) THEN
              sum = sum + term
              xn = xn + ONE
              xtons = -xtons * x
              term = xtons / xn
              GO TO 10
          END IF
          dlog1p = sum
      ELSE
          dlog1p = dlog(ONE + x)
      END IF
C
      END

      DOUBLE PRECISION FUNCTION dl1p10(x)
C     (log1p10)
C     Return dlog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION dlog1p
      DOUBLE PRECISION LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366D+00/
C
      dl1p10 = dlog1p(x) * LOG10E
C
      END
