      REAL FUNCTION alog1p(x)
C     (log1p)
C     Return alog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses a [3,3]-degree rational minimax polynomial
C     computed in Maple by
C
C         with(numapprox):
C         Digits := 35:
C         minimax((ln(1+x)-x)/x^2, x = -0.5..0.5, [3,3], 1, 'err');
C         printf("%.2e\n", err);
C
C     The reported absolute error is 1.20e-09, below the IEEE 754
C     machine epsilon of 2**(-23) = 9.02e-09.  This is the lowest
C     degree, and thus optimal, minimax polynomial whose error is below
C     that limit.
C     (24-Jun-2002)
      REAL x
      REAL CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = -1/beta, for arbitrary base beta
C
      DATA CUTLO / -0.5e+00 /
C
C     CUTHI = 1/beta, for arbitrary base beta
C
      DATA CUTHI / +0.5e+00 /
C
      DATA ONE   / 1.0e+00 /
      DATA ZERO  / 0.0e+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  log(1+x)
C     x in [CUTLO, CUTHI]:      minimax polynomial
C     x in (CUTHI, Infinity]:   log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of log1p(NaN) and log1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          alog1p = x
      ELSE IF (x .eq. ZERO) THEN
          alog1p = x
      ELSE IF (x .lt. CUTLO) THEN
          alog1p = alog(ONE + x)
      ELSE IF (x .le. CUTHI) THEN
          alog1p = x + (x * x * (
     X ( -4.36853891e-01 +
     X ( -5.55650140e-01 +
     X ( -1.59005713e-01 +
     X ( -1.08045674e-03
     X ) * x) * x) * x)
     X /
     X (  8.73707794e-01 +
     X (  1.69377203e+00 +
     X (  1.01033764e+00 +
     X (  1.78318685e-01
     X ) * x) * x) * x) ) )
      ELSE
          alog1p = alog(ONE + x)
      END IF
C
      END

      REAL FUNCTION al1p10(x)
C     (log1p10)
C     Return alog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      REAL x
      REAL alog1p
      REAL LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366E+00/
C
      al1p10 = alog1p(x) * LOG10E
C
      END
