      DOUBLE PRECISION FUNCTION dexpm1(x)
C     (dexpm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses a [8,2]-degree rational minimax polynomial
C     computed in Maple by
C
C         with(numapprox):
C         Digits := 60:
C         minimax((exp(x)-1-x)/x^2, x = ln(1/2)..ln(3/2),
C                 [8,2], 1, 'err');
C         printf("%.2e\n", err);
C
C     The reported absolute error is 4.67e-18, below the IEEE 754
C     machine epsilon of 2**(-52) = 2.22e-16.  This is the lowest
C     degree, and thus optimal, minimax polynomial whose error is below
C     that limit.
C     (28-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603D0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625D0/
C
      DATA ZERO / 0.0D+00 /
      DATA ONE  / 1.0D+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  exp(x) - 1
C     x in [CUTLO, CUTHI]:      minimax polynomial
C     x in (CUTHI, Infinity]:   exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .ne. x) THEN
          dexpm1 = x
      ELSE IF (x .eq. ZERO) THEN
          dexpm1 = x
      ELSE IF (x .lt. CUTLO) THEN
          dexpm1 = dexp(x) - ONE
      ELSE IF (x .le. CUTHI) THEN
          dexpm1 = x + (x * x * (
     X ( -4.6725946288043156d-01 +
     X (  3.2851354138135712d-02 +
     X ( -7.7147172221884448d-03
     X ) * x) * x)
     X /
     X ( -9.3451892576086312d-01 +
     X (  3.7720901686322570d-01 +
     X ( -6.3289196252045559d-02 +
     X (  5.2376294414332701d-03 +
     X ( -1.6270783567142839d-04 +
     X ( -4.3752185414398622d-06 +
     X (  1.9587930519571511d-07 +
     X (  1.6698937101747154d-08
     X ) * x) * x) * x) * x) * x) * x) * x) ) )
      ELSE
          dexpm1 = dexp(x) - ONE
      ENDIF
C
      END
