      DOUBLE PRECISION FUNCTION dexpm1(x)
C     (dexpm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses a [4,7]-degree Pad approximation computed in
C     Maple by
C
C         with(numapprox):
C         Digits := 60:
C         pade((exp('x') - 1 - 'x')/'x'^2,
C             'x' = (ln(1/2)+ln(3/2))/2, [4,7]);
C
C     The computed relative error is 4.68e-18 at x = 0.4055, below the
C     IEEE 754 machine epsilon of 2**(-52) = 2.22e-16.  This is the
C     lowest degree, and thus optimal, Pad approximation whose error
C     is below that limit.
C     (29-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603D0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625D0/
C
      DATA ZERO / 0.0D+00 /
      DATA ONE  / 1.0D+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  exp(x) - 1
C     x in [CUTLO, CUTHI]:      Pad approximation
C     x in (CUTHI, Infinity]:   exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .ne. x) THEN
          dexpm1 = x
      ELSE IF (x .eq. ZERO) THEN
          dexpm1 = x
      ELSE IF (x .lt. CUTLO) THEN
          dexpm1 = dexp(x) - ONE
      ELSE IF (x .le. CUTHI) THEN
          dexpm1 = x + (x * x * (
     X (  4.6969426863446711d-01 +
     X ( -4.8268560688693705d-02 +
     X (  1.0918155925073419d-02 +
     X ( -3.9886659000102040d-04 +
     X (  3.3477940659025022d-05
     X ) * x) * x) * x) * x)
     X /
     X (  9.3938853726893422d-01 +
     X ( -4.0966663380036548d-01 +
     X (  8.0109478344524148d-02 +
     X ( -9.0184820992952034d-03 +
     X (  6.1569134552071340d-04 +
     X ( -2.3658603381660569d-05 +
     X (  3.2976614358014640d-07 +
     X (  5.3290461261147527d-09
     X ) * x) * x) * x) * x) * x) * x) * x) ) )
      ELSE
          dexpm1 = dexp(x) - ONE
      ENDIF
C
      END
