      REAL FUNCTION expm1(x)
C     (expm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses a [4,4]-degree Pad approximation computed in
C     Maple by
C
C         with(numapprox):
C         Digits := 20:
C         pade((exp('x') - 1)/'x', 'x' = 0, [4,4]):
C
C     The coefficients in this approximation are exactly representable.
C     The computed relative error is 6.08e-11, below the IEEE 754
C     machine epsilon of 2**(-23) = 1.19e-07.  This is the lowest
C     degree, and thus optimal, Pad approximation whose error is below
C     that limit.
C     (29-Jun-2002)
      REAL x
      REAL CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603E0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625E0/
C
      DATA ZERO / 0.0E+00 /
      DATA ONE  / 1.0E+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  exp(x) - 1
C     x in [CUTLO, CUTHI]:      Pad approximation
C     x in (CUTHI, Infinity]:   exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .ne. x) THEN
          expm1 = x
      ELSE IF (x .eq. ZERO) THEN
          expm1 = x
      ELSE IF (x .lt. CUTLO) THEN
          expm1 = exp(x) - ONE
      ELSE IF (x .le. CUTHI) THEN
          expm1 = x * (( 15120.0 + ( 840.0 + ( 420.0 + ( 20.0 + ( 1.0 )
     X        * x) * x) * x) * x) /
     X        ( 15120.0 + (-6720.0 + ( 1260.0 + (-120.0 +
     X        ( 5.0 ) * x) * x) * x) * x) )
      ELSE
          expm1 = exp(x) - ONE
      ENDIF
C
      END
