      REAL*16 FUNCTION qlog1p(x)
C     (qlog1p)
C     Return qlog(1 + x), taking care to avoid subtraction loss.
C
C     This version incorporates run-time precomputation of a table
C     of reciprocals to replace division by multiplication in the
C     Taylor series computation.
C     (17-Jun-2002)
      REAL*16 x
      REAL*16 CUTLO, CUTHI, ONE, ZERO
      REAL*16 xninv(117)
      REAL*16 sum, term, xtons
      INTEGER n, MAXN
      LOGICAL first
C
      SAVE first, xninv
C
      DATA first / .TRUE. /
C
C     The maximum number of Taylor series terms (20, 48, 59, 107) is
C     known in advance, but we compute 10 more reciprocals for safety.
C
      DATA MAXN / 117 /
C
C     CUTLO = -1/beta, for arbitrary base beta
C
      DATA CUTLO / -0.5q+00 /
C
C     CUTHI = 1/beta, for arbitrary base beta
C
      DATA CUTHI / +0.5q+00 /
C
      DATA ONE   / 1.0q+00 /
      DATA ZERO  / 0.0q+00 /
C
      IF (first) THEN
          DO 5 n = 1, MAXN
              xninv(n) = ONE / QFLOAT(n)
    5     CONTINUE
          first = .FALSE.
      END IF
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  log(1+x)
C     x in [CUTLO, CUTHI]:      Taylor series
C     x in (CUTHI, Infinity]:   log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of qlog1p(NaN) and qlog1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          qlog1p = x
      ELSE IF (x .eq. ZERO) THEN
          qlog1p = x
      ELSE IF (x .lt. CUTLO) THEN
          qlog1p = qlog(ONE + x)
      ELSE IF (x .le. CUTHI) THEN
          term = x
          sum = ZERO
          n = 1
          xtons = x
   10     IF ((sum + term) .ne. sum) THEN
              sum = sum + term
              n = n + 1
              xtons = -xtons * x
              term = xtons * xninv(n)
              GO TO 10
          END IF
          qlog1p = sum
      ELSE
          qlog1p = qlog(ONE + x)
      END IF
C
      END

      REAL*16 FUNCTION ql1p10(x)
C     (log1p10)
C     Return qlog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      REAL*16 x
      REAL*16 qlog1p
      REAL*16 LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366Q+00/
C
      ql1p10 = qlog1p(x) * LOG10E
C
      END
